
package zoomarket;

import java.io.Serializable;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;


public class Product implements Serializable {

    private int id;
    private int quantity;
    private double price;
    private double vipPrice;
    private ProductSpec spec;
    private LinkedList<Review> reviews;


    public Product( int id, int quantity, double price, double vipPrice, ProductSpec spec ) throws ZooException
    {
        setId( id );
        setQuantity( quantity );
        setPrice( price );
        setVipPrice( vipPrice );
        setProductSpec( spec );
        reviews = new LinkedList<Review>();
    }

    public void setId( int id ) throws ZooException{
        if( id < -1 ) throw new ZooException("Invalid Product Id");
        if( id == -1 )
        {
            // Get id from database
        }
        else this.id = id;
    }

    public void setQuantity( int quantity ) throws ZooException{
        if( id < 0 ) throw new ZooException("Invalid Product Quantity");
        this.quantity = quantity;
    }

    public void setPrice( double price ) throws ZooException{
        if( price < 0 ) throw new ZooException("Invalid Product Price");
        this.price = price;
    }

    public void setVipPrice( double vipPrice ) throws ZooException{
        if( vipPrice < 0 ) throw new ZooException("Invalid Product Vip Price");
        this.vipPrice = vipPrice;
    }

    public void setProductSpec( ProductSpec spec ) throws ZooException{
        if( spec == null ) throw new ZooException("Empty Product Specification");
        this.spec = spec;
    }

    public int getId(){
        return id;
    }

    public int getQuantity(){
        return quantity;
    }

    public double getPrice(){
        return price;
    }

    public double getVipPrice(){
        return vipPrice;
    }

    public ProductSpec getSpec(){
        return spec;
    }

	public String getProductName(){
		return spec.getName();
	}

	public String getProductDescription(){
        StringBuffer buffer = new StringBuffer();
        buffer.append( "Product Id:\t" );
        buffer.append( id );
        buffer.append( "\n" + spec.toString() );
        buffer.append( "Quantity:\t" );
        buffer.append( quantity );
        buffer.append( "\nPrice:\t" );
        buffer.append( price );
        buffer.append( "\nVip price:\t" );
        buffer.append( vipPrice );
        return buffer.toString();
	}

	public String getProductReviews(){
        return reviews.toString();
	}

    public void addReview(String username, String content) throws ZooException{
        reviews.add( new Review(username, content, new Date()) );
    }

	public Review removeReview(int index) throws ZooException{
        try{
            return reviews.remove( index );
        }
        catch( IndexOutOfBoundsException ex ){
            throw new ZooException( ex, "Error while removing review: Index " + index
                    + " is out of bounds. There are " + reviews.size() + " available users" );
        }
	}

    public LinkedList<Review> getReviews(){
        return (LinkedList<Review>) reviews.clone();
    }

    public LinkedList<Review> getReviews(String username){
        LinkedList<Review> result = new LinkedList<Review>();
        Iterator iter = reviews.iterator();

        while( iter.hasNext() ){
            Review rev = (Review) iter.next();
            if( rev.getName().equals( username )) {
                result.add( rev );
            }
        }

        return result;
    }

    public LinkedList<Review> getReviews(Date date){
        LinkedList<Review> result = new LinkedList<Review>();
        Iterator iter = reviews.iterator();

        while( iter.hasNext() ){
            Review rev = (Review) iter.next();
            if( rev.getDate().equals( date )) {
                result.add( rev );
            }
        }

        return result;
    }

    public String toString()
    {
        StringBuffer buffer = new StringBuffer();
        buffer.append( "============\n" );
        buffer.append( getProductDescription() );
        buffer.append( "\nReviews:\n" );
        buffer.append( getProductReviews() );
        buffer.append( "\n============\n" );
        return buffer.toString();
    }

}
