
package zoomarket;

import java.io.Serializable;


public class ProductSpec implements Serializable {

    String name;
    Manufacturer manufacturer;
    ProductType type;
    int description;


    public ProductSpec( String name, Manufacturer manufacturer,
            ProductType productType, int description ) throws ZooException{
        setName( name );
        setDescription( description );
        setManufacturer( manufacturer );
        setProductType( productType );
    }

    public void setName( String name ) throws ZooException{
        if( name == null || name.equals("") ) throw new ZooException("Empty Product Name");
        this.name = name;
    }

    public void setManufacturer( Manufacturer manufacturer ) throws ZooException{
        if( manufacturer == null ) throw new ZooException("Empty Product Manufacturer");
        this.manufacturer = manufacturer;
    }

    public void setProductType( ProductType type ) throws ZooException{
        if( type == null ) throw new ZooException("Empty Product Type");
        this.type = type;
    }

    public void setDescription( int description ) throws ZooException{
        if( description < 0 && description > 2 ) throw new ZooException("Invalid Product Description");
        this.description = description;
    }


    public String getName(){
        return name;
    }

    public Manufacturer getManufacturer(){
        return manufacturer;
    }

    public ProductType getProductType(){
        return type;
    }

    public int getDescription(){
        return description;
    }

	public String getDescriptionAsString(){
		String strDescription = "";
		switch(description) {
			case 0: {strDescription = "Pack"; break;}
			case 1: {strDescription = "Kilogram"; break;}
			case 2: {strDescription = "Litre"; break;}
			default: strDescription = "";
		}
		return strDescription;
	}


    public boolean matches( ProductSpec other ){
        return name.equals(other.name) && description == other.description
                && manufacturer.equals(other.manufacturer) && type == other.type;
    }

    public String toString()
    {
        StringBuffer buffer = new StringBuffer();
        buffer.append( "Product Name:\t" );
        buffer.append( name );
        buffer.append( "\nManufacturer:\t" );
        buffer.append( manufacturer.toString() );
        buffer.append( "\nProduct type:\t" );
        buffer.append( type.toString() );
        buffer.append( "\nDescription:\t" );
        buffer.append( getDescriptionAsString() );
        buffer.append( "\n" );
        return buffer.toString();
    }

}
